@@ -73,7 +73,7 @@ class PerturbFlow(nn.Module):
                  config_enum: str = 'parallel',
                  use_cuda: bool = False,
                  seed: int = 42,
-                 zero_bias: bool = True,
+                 zero_bias: bool|list = True,
                  dtype = torch.float32, # type: ignore
                  ):
         super().__init__()
@@ -97,7 +97,11 @@ class PerturbFlow(nn.Module):
         self.post_layer_fct = post_layer_fct
         self.post_act_fct = post_act_fct
         self.hidden_layer_activation = hidden_layer_activation
-        self.use_bias = not zero_bias
+        if type(zero_bias) == list:
+            self.use_bias = [not x for x in zero_bias]
+        else:
+            self.use_bias = [not zero_bias] * self.cell_factor_size
+        #self.use_bias = not zero_bias
         
         self.codebook_weights = None
 
@@ -198,7 +202,7 @@ class PerturbFlow(nn.Module):
         if self.cell_factor_size>0:
             self.cell_factor_effect = nn.ModuleList()
             for i in np.arange(self.cell_factor_size):
-                if self.use_bias:
+                if self.use_bias[i]:
                     self.cell_factor_effect.append(MLP(
                         [self.latent_dim+1] + self.decoder_hidden_layers + [self.latent_dim],
                         activation=activate_fct,
@@ -207,7 +211,6 @@ class PerturbFlow(nn.Module):
                         post_act_fct=post_act_fct,
                         allow_broadcast=self.allow_broadcast,
                         use_cuda=self.use_cuda,
-                        bias=True,
                         )
                     )
                 else:
@@ -424,12 +427,13 @@ class PerturbFlow(nn.Module):
                 zns = pyro.sample('zn', dist.Gumbel(zn_loc, zn_scale).to_event(1))
 
             if self.cell_factor_size>0:
-                zus = None
-                for i in np.arange(self.cell_factor_size):
-                    if i==0:
-                        zus = self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
-                    else:
-                        zus = zus + self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+                #zus = None
+                #for i in np.arange(self.cell_factor_size):
+                #    if i==0:
+                #        zus = self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+                #    else:
+                #        zus = zus + self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+                zus = self._total_effects(zns, us)
                 zs = zns+zus
             else:
                 zs = zns
@@ -619,12 +623,13 @@ class PerturbFlow(nn.Module):
                     zns = pyro.sample('zn', dist.Gumbel(zn_loc, zn_scale).to_event(1), obs=embeds)
 
             if self.cell_factor_size>0:
-                zus = None
-                for i in np.arange(self.cell_factor_size):
-                    if i==0:
-                        zus = self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
-                    else:
-                        zus = zus + self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+                #zus = None
+                #for i in np.arange(self.cell_factor_size):
+                #    if i==0:
+                #        zus = self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+                #    else:
+                #        zus = zus + self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+                zus = self._total_effects(zns, us)
                 zs = zns+zus
             else:
                 zs = zns
@@ -661,6 +666,15 @@ class PerturbFlow(nn.Module):
                 zn_loc, zn_scale = self.encoder_zn(xs)
                 zns = pyro.sample('zn', dist.Normal(zn_loc, zn_scale).to_event(1))
 
+    def _total_effects(self, zns, us):
+        zus = None
+        for i in np.arange(self.cell_factor_size):
+            if i==0:
+                zus = self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+            else:
+                zus = zus + self.cell_factor_effect[i]([zns,us[:,i].reshape(-1,1)])
+        return zus
+                        
     def _get_codebook(self):
         I = torch.eye(self.code_size, **self.options)
         if self.latent_dist=='studentt':
@@ -717,7 +731,8 @@ class PerturbFlow(nn.Module):
         if self.supervised_mode:
             alpha = self.encoder_n(xs)
         else:
-            zns,_ = self.encoder_zn(xs)
+            #zns,_ = self.encoder_zn(xs)
+            zns = self._get_basal_embedding(xs)
             alpha = self.encoder_n(zns)
         return alpha
     
@@ -787,7 +802,8 @@ class PerturbFlow(nn.Module):
         return A
     
     def _cell_response(self, xs, factor_idx, perturb):
-        zns,_ = self.encoder_zn(xs)       
+        #zns,_ = self.encoder_zn(xs)    
+        zns = self._get_basal_embedding(xs)   
         if perturb.ndim==2:
             ms = self.cell_factor_effect[factor_idx]([zns, perturb])
         else:
