@@ -209,3 +209,36 @@ class MLP(nn.Module):
     # pass through our sequential for the output!
     def forward(self, *args, **kwargs):
         return self.sequential_mlp.forward(*args, **kwargs)
+
+
+
+
+class ZeroBiasMLP(nn.Module):
+    def __init__(
+        self,
+        mlp_sizes,
+        activation=nn.ReLU,
+        output_activation=None,
+        post_layer_fct=lambda layer_ix, total_layers, layer: None,
+        post_act_fct=lambda layer_ix, total_layers, layer: None,
+        allow_broadcast=False,
+        use_cuda=False,
+    ):
+        # init the module object
+        super().__init__()
+        self.mlp = MLP(mlp_sizes=mlp_sizes,
+                       activation=activation,
+                       output_activation=output_activation,
+                       post_layer_fct=post_layer_fct,
+                       post_act_fct=post_act_fct,
+                       allow_broadcast=allow_broadcast,
+                       use_cuda=use_cuda,
+                       bias=True)
+        
+    # pass through our sequential for the output!
+    def forward(self, x):
+        y = self.mlp(x)
+        mask = torch.zeros_like(y)
+        mask[x[1][:,0]>0,:] = 1
+        return y*mask
+        
\ No newline at end of file
